@@ -174,6 +174,7 @@ save_file_when_idle (gpointer user_data)
 
 	buf = icalcomponent_as_ical_string (priv->icalcomp);
 	result = gnome_vfs_write (handle, buf, strlen (buf) * sizeof (char), &out);
+	g_free (buf);
 	gnome_vfs_close (handle);
 	if (result != GNOME_VFS_OK) {
 		gnome_vfs_uri_unref (uri);
@@ -200,12 +201,14 @@ save_file_when_idle (gpointer user_data)
 	g_static_rec_mutex_unlock (&priv->idle_save_rmutex);
 	e_cal_backend_notify_error (E_CAL_BACKEND (cbfile),
 				  _("Cannot save calendar data: Malformed URI."));
-	return TRUE;
+	return FALSE;
 
  error:
 	g_static_rec_mutex_unlock (&priv->idle_save_rmutex);
+	/* TODO Add concat the message "Cannot save calendar data" to the error string later.Not
+	   doing it now as we have string freeze. */
 	e_cal_backend_notify_error (E_CAL_BACKEND (cbfile), gnome_vfs_result_to_string (result));
-	return TRUE;
+	return FALSE;
 }
 
 static void
@@ -470,12 +473,13 @@ add_component (ECalBackendFile *cbfile, ECalComponent *comp, gboolean add_to_top
 
 	obj_data = g_hash_table_lookup (priv->comp_uid_hash, uid);
 	if (e_cal_component_is_instance (comp)) {
-		const char *rid;
+		char *rid;
 
 		rid = e_cal_component_get_recurid_as_string (comp);
 		if (obj_data) {
 			if (g_hash_table_lookup (obj_data->recurrences, rid)) {
 				g_warning (G_STRLOC ": Tried to add an already existing recurrence");
+				g_free (rid);
 				return;
 			}
 		} else {
@@ -485,7 +489,7 @@ add_component (ECalBackendFile *cbfile, ECalComponent *comp, gboolean add_to_top
 			g_hash_table_insert (priv->comp_uid_hash, g_strdup (uid), obj_data);
 		}
 
-		g_hash_table_insert (obj_data->recurrences, g_strdup (rid), comp);
+		g_hash_table_insert (obj_data->recurrences, rid, comp);
 		obj_data->recurrences_list = g_list_append (obj_data->recurrences_list, comp);
 	} else {
 		/* Ensure that the UID is unique; some broken implementations spit
@@ -520,8 +524,6 @@ add_component (ECalBackendFile *cbfile, ECalComponent *comp, gboolean add_to_top
 		g_assert (icalcomp != NULL);
 
 		icalcomponent_add_component (priv->icalcomp, icalcomp);
-
-		save (cbfile);
 	}
 }
 
@@ -735,6 +737,7 @@ notify_removals_cb (gpointer key, gpointer value, gpointer data)
 		e_cal_backend_notify_object_removed (context->backend, id, old_obj_str, NULL);
 
 		e_cal_component_free_id (id);
+		g_free (old_obj_str);
 		g_object_unref (comp);
 	}
 }
@@ -763,6 +766,7 @@ notify_adds_modifies_cb (gpointer key, gpointer value, gpointer data)
 			return;
 
 		e_cal_backend_notify_object_created (context->backend, new_obj_str);
+		g_free (new_obj_str);
 	} else {
 		old_icomp = e_cal_component_get_icalcomponent (old_obj_data->full_object);
 		new_icomp = e_cal_component_get_icalcomponent (new_obj_data->full_object);
@@ -779,6 +783,8 @@ notify_adds_modifies_cb (gpointer key, gpointer value, gpointer data)
 
 			e_cal_backend_notify_object_modified (context->backend, old_obj_str, new_obj_str);
 		}
+		g_free (old_obj_str);
+		g_free (new_obj_str);
 	}
 }
 
@@ -1122,7 +1128,7 @@ e_cal_backend_file_get_object (ECalBackendSync *backend, EDataCal *cal, const ch
 				return GNOME_Evolution_Calendar_ObjectNotFound;
                         }
 
-			*object = g_strdup (icalcomponent_as_ical_string (icalcomp));
+			*object = icalcomponent_as_ical_string (icalcomp);
 
 			icalcomponent_free (icalcomp);
 		}
@@ -1139,7 +1145,7 @@ e_cal_backend_file_get_object (ECalBackendSync *backend, EDataCal *cal, const ch
 			/* add all detached recurrences */
 			g_hash_table_foreach (obj_data->recurrences, (GHFunc) add_detached_recur_to_vcalendar, icalcomp);
 
-			*object = g_strdup (icalcomponent_as_ical_string (icalcomp));
+			*object = icalcomponent_as_ical_string (icalcomp);
 
 			icalcomponent_free (icalcomp);
 		} else
@@ -1186,7 +1192,7 @@ e_cal_backend_file_get_timezone (ECalBackendSync *backend, EDataCal *cal, const
 		return GNOME_Evolution_Calendar_InvalidObject;
 	}
 
-	*object = g_strdup (icalcomponent_as_ical_string (icalcomp));
+	*object = icalcomponent_as_ical_string (icalcomp);
 
 	g_static_rec_mutex_unlock (&priv->idle_save_rmutex);
 	return GNOME_Evolution_Calendar_Success;
@@ -1534,7 +1540,7 @@ e_cal_backend_file_get_free_busy (ECalBackendSync *backend, EDataCal *cal, GList
 		if (e_cal_backend_mail_account_get_default (&address, &name)) {
 			vfb = create_user_free_busy (cbfile, address, name, start, end);
 			calobj = icalcomponent_as_ical_string (vfb);
-			*freebusy = g_list_append (*freebusy, g_strdup (calobj));
+			*freebusy = g_list_append (*freebusy, calobj);
 			icalcomponent_free (vfb);
 			g_free (address);
 			g_free (name);
@@ -1545,7 +1551,7 @@ e_cal_backend_file_get_free_busy (ECalBackendSync *backend, EDataCal *cal, GList
 			if (e_cal_backend_mail_account_is_valid (address, &name)) {
 				vfb = create_user_free_busy (cbfile, address, name, start, end);
 				calobj = icalcomponent_as_ical_string (vfb);
-				*freebusy = g_list_append (*freebusy, g_strdup (calobj));
+				*freebusy = g_list_append (*freebusy, calobj);
 				icalcomponent_free (vfb);
 				g_free (name);
 			}
@@ -1718,6 +1724,9 @@ e_cal_backend_file_internal_get_timezone (ECalBackend *backend, const char *tzid
 		zone = icalcomponent_get_timezone (priv->icalcomp, tzid);
 		if (!zone)
 			zone = icaltimezone_get_builtin_timezone_from_tzid (tzid);
+
+		if (!zone && E_CAL_BACKEND_CLASS (parent_class)->internal_get_timezone)
+			zone = E_CAL_BACKEND_CLASS (parent_class)->internal_get_timezone (backend, tzid);
 	}
 
 	g_static_rec_mutex_unlock (&priv->idle_save_rmutex);
@@ -1895,7 +1904,8 @@ e_cal_backend_file_modify_object (ECalBackendSync *backend, EDataCal *cal, const
 	ECalBackendFile *cbfile;
 	ECalBackendFilePrivate *priv;
 	icalcomponent *icalcomp;
-	const char *comp_uid, *rid = NULL;
+	const char *comp_uid;
+	char *rid = NULL;
 	char *real_rid;
 	ECalComponent *comp, *recurrence;
 	ECalBackendFileObject *obj_data;
@@ -1965,6 +1975,7 @@ e_cal_backend_file_modify_object (ECalBackendSync *backend, EDataCal *cal, const
 			save (cbfile);
 
 			g_static_rec_mutex_unlock (&priv->idle_save_rmutex);
+			g_free (rid);
 			return GNOME_Evolution_Calendar_Success;
 		}
 
@@ -1982,16 +1993,16 @@ e_cal_backend_file_modify_object (ECalBackendSync *backend, EDataCal *cal, const
 
 		/* add the detached instance */
 		g_hash_table_insert (obj_data->recurrences,
-				     g_strdup (rid),
+				     rid,
 				     comp);
 		icalcomponent_add_component (priv->icalcomp,
 					     e_cal_component_get_icalcomponent (comp));
 		priv->comp = g_list_append (priv->comp, comp);
 		obj_data->recurrences_list = g_list_append (obj_data->recurrences_list, comp);
+		rid = NULL;
 		break;
 	case CALOBJ_MOD_THISANDPRIOR :
 	case CALOBJ_MOD_THISANDFUTURE :
-		rid = e_cal_component_get_recurid_as_string (comp);
 		if (!rid || !*rid) {
 			if (old_object)
 				*old_object = e_cal_component_get_as_string (obj_data->full_object);
@@ -2000,6 +2011,8 @@ e_cal_backend_file_modify_object (ECalBackendSync *backend, EDataCal *cal, const
 
 			/* Add the new object */
 			add_component (cbfile, comp, TRUE);
+			g_free (rid);
+			rid = NULL;
 			break;
 		}
 
@@ -2040,12 +2053,13 @@ e_cal_backend_file_modify_object (ECalBackendSync *backend, EDataCal *cal, const
 
 		/* add the new detached recurrence */
 		g_hash_table_insert (obj_data->recurrences,
-				     g_strdup (e_cal_component_get_recurid_as_string (comp)),
+				     rid,
 				     comp);
 		icalcomponent_add_component (priv->icalcomp,
 					     e_cal_component_get_icalcomponent (comp));
 		priv->comp = g_list_append (priv->comp, comp);
 		obj_data->recurrences_list = g_list_append (obj_data->recurrences_list, comp);
+		rid = NULL;
 		break;
 	case CALOBJ_MOD_ALL :
 		/* in this case, we blow away all recurrences, and start over
@@ -2117,6 +2131,7 @@ e_cal_backend_file_modify_object (ECalBackendSync *backend, EDataCal *cal, const
 	}
 
 	save (cbfile);
+	g_free (rid);
 
 	g_static_rec_mutex_unlock (&priv->idle_save_rmutex);
 	return GNOME_Evolution_Calendar_Success;
@@ -2276,7 +2291,7 @@ cancel_received_object (ECalBackendFile *cbfile, icalcomponent *icalcomp)
 {
 	ECalBackendFileObject *obj_data;
 	ECalBackendFilePrivate *priv;
-	const char *rid;
+	char *rid;
 	ECalComponent *comp;
 
 	priv = cbfile->priv;
@@ -2299,6 +2314,8 @@ cancel_received_object (ECalBackendFile *cbfile, icalcomponent *icalcomp)
 	else
 		remove_component (cbfile, icalcomponent_get_uid (icalcomp), obj_data);
 
+	g_free (rid);
+
 	return TRUE;
 }
 
@@ -2485,8 +2502,8 @@ e_cal_backend_file_receive_objects (ECalBackendSync *backend, EDataCal *cal, con
 
 	/* Now we manipulate the components we care about */
 	for (l = comps; l; l = l->next) {
-		const char *uid, *rid;
-		char *object, *old_object;
+		const char *uid;
+		char *object, *old_object, *rid;
 		ECalBackendFileObject *obj_data;
 
 		subcomp = l->data;
@@ -2535,24 +2552,28 @@ e_cal_backend_file_receive_objects (ECalBackendSync *backend, EDataCal *cal, con
 				e_cal_backend_notify_object_created (E_CAL_BACKEND (backend), object);
 				g_free (object);
 			}
+			g_free (rid);
 			break;
 		case ICAL_METHOD_ADD:
 			/* FIXME This should be doable once all the recurid stuff is done */
 			status = GNOME_Evolution_Calendar_UnsupportedMethod;
+			g_free (rid);
 			goto error;
 			break;
 		case ICAL_METHOD_COUNTER:
 			status = GNOME_Evolution_Calendar_UnsupportedMethod;
+			g_free (rid);
 			goto error;
 			break;
 		case ICAL_METHOD_DECLINECOUNTER:
 			status = GNOME_Evolution_Calendar_UnsupportedMethod;
+			g_free (rid);
 			goto error;
 			break;
 		case ICAL_METHOD_CANCEL:
 			if (cancel_received_object (cbfile, subcomp)) {
 				ECalComponentId *id;
-				object = (char *) icalcomponent_as_ical_string (subcomp);
+				object =  icalcomponent_as_ical_string (subcomp);
 				obj_data = g_hash_table_lookup (priv->comp_uid_hash, uid);
 				if (obj_data)
 					old_object = e_cal_component_get_as_string (obj_data->full_object);
@@ -2569,10 +2590,13 @@ e_cal_backend_file_receive_objects (ECalBackendSync *backend, EDataCal *cal, con
 				e_cal_component_free_id (id);
 
 				g_free (old_object);
+				g_free (object);
 			}
+			g_free (rid);
 			break;
 		default:
 			status = GNOME_Evolution_Calendar_UnsupportedMethod;
+			g_free (rid);
 			goto error;
 		}
 	}
