@@ -116,6 +116,10 @@ e_cal_backend_google_internal_get_timezone (ECalBackend *backend, const char *tz
 	icaltimezone *zone;
 
 	zone = icaltimezone_get_builtin_timezone_from_tzid (tzid);
+
+	if (!zone && E_CAL_BACKEND_CLASS (parent_class)->internal_get_timezone)
+		zone = E_CAL_BACKEND_CLASS (parent_class)->internal_get_timezone (backend, tzid);
+
 	if (!zone)
 		return icaltimezone_get_utc_timezone ();
 
@@ -540,7 +544,7 @@ static receive_object (ECalBackendGoogle *cbgo, EDataCal *cal, icalcomponent *ic
 	GSList *comps = NULL, *l = NULL;
 	icalproperty_method method;
 	icalproperty *icalprop;
-	gboolean instances = TRUE, found;
+	gboolean instances = TRUE, found = FALSE;
 	icalparameter_partstat pstatus = 0;
 
 	priv = cbgo->priv;
@@ -693,12 +697,11 @@ send_object (ECalBackendGoogle *cbgo, EDataCal *cal, icalcomponent *icalcomp, ic
 	ECalComponent *comp, *found_comp = NULL;
 	ECalBackendGooglePrivate *priv;
 	ECalBackendSyncStatus status = GNOME_Evolution_Calendar_OtherError;
-	const char *uid, *rid;
+	const char *uid;
 
 	priv = cbgo->priv;
 	comp = e_cal_component_new ();
 	e_cal_component_set_icalcomponent (comp, icalcomponent_new_clone (icalcomp));	
-	rid = e_cal_component_get_recurid_as_string (comp);	
 
 	e_cal_component_get_uid (comp, (const char **)&uid);
 	found_comp = e_cal_backend_cache_get_component (priv->cache, uid, NULL);
@@ -885,6 +888,7 @@ e_cal_backend_google_modify_object (ECalBackendSync *backend, EDataCal *cal, con
 	const char *uid=NULL, *rid=NULL;
 	GDataEntry *entry, *entry_from_server=NULL;
 	gchar *edit_link;
+	GSList *l;
 
 	*old_object = NULL;
 	cbgo = E_CAL_BACKEND_GOOGLE (backend);
@@ -907,7 +911,6 @@ e_cal_backend_google_modify_object (ECalBackendSync *backend, EDataCal *cal, con
 	e_cal_component_get_uid (comp, &uid);
 
 	/* Check if object exists */
-	GSList *l;
 	switch (priv->mode) {
 		case CAL_MODE_ANY:
 		case CAL_MODE_REMOTE:
@@ -1071,13 +1074,13 @@ e_cal_backend_google_create_object (ECalBackendSync *backend, EDataCal *cal, cha
 	switch (priv->mode) {
 
 		case CAL_MODE_ANY:
-		case CAL_MODE_REMOTE:
+		case CAL_MODE_REMOTE: {
 			/* Create an appointment */
+			GDataEntry *updated_entry;
 
 			item = e_go_item_from_cal_component (cbgo, comp);
 			entry = e_go_item_get_entry (item);
 
-			GDataEntry *updated_entry;
 			updated_entry = gdata_service_insert_entry (GDATA_SERVICE(priv->service), priv->uri, entry);
 			if (!GDATA_IS_ENTRY (updated_entry)) {
 				g_message ("\n Entry Insertion Failed %s \n", G_STRLOC);
@@ -1086,7 +1089,7 @@ e_cal_backend_google_create_object (ECalBackendSync *backend, EDataCal *cal, cha
 			id = gdata_entry_get_id (updated_entry);
 			e_cal_component_set_uid (comp, id);
 
-			break;
+			break; }
 		default:
 			break;
 	}
