@@ -323,9 +323,9 @@ retrieval_done (SoupSession *session, SoupMessage *msg, ECalBackendHttp *cbhttp)
 
 		subcomp_kind = icalcomponent_isa (subcomp);
 		prop = icalcomponent_get_first_property (subcomp, ICAL_UID_PROPERTY);
-		if (!prop) {
+		if (!prop && subcomp_kind == kind) {
 			g_warning (" The component does not have the  mandatory property UID \n");
-			subcomp = icalcomponent_get_next_component (icalcomp, kind);
+			subcomp = icalcomponent_get_next_component (icalcomp, ICAL_ANY_COMPONENT);
 			continue;
 		}
 
@@ -365,7 +365,7 @@ retrieval_done (SoupSession *session, SoupMessage *msg, ECalBackendHttp *cbhttp)
 			icaltimezone_free (zone, 1);
 		}
 
-		subcomp = icalcomponent_get_next_component (icalcomp, kind);
+		subcomp = icalcomponent_get_next_component (icalcomp, ICAL_ANY_COMPONENT);
 	}
 
 	e_file_cache_thaw_changes (E_FILE_CACHE (priv->cache));
