@@ -31,14 +31,14 @@
 #include <sys/types.h>
 #include <time.h>
 
-#ifndef G_OS_WIN32
-#include <sys/wait.h>
-#endif
-
 #include <glib.h>
 #include <glib/gstdio.h>
 #include <glib/gi18n-lib.h>
 
+#ifndef G_OS_WIN32
+#include <sys/wait.h>
+#endif
+
 #include <libedataserver/e-sexp.h>
 #include <libedataserver/e-memory.h>
 #include <libedataserver/e-msgport.h>
@@ -590,11 +590,26 @@ do_label (struct _ESExp *f, int argc, struct _ESExpResult **argv, CamelFilterDri
 
 	d(fprintf (stderr, "setting label tag\n"));
 	if (argc > 0 && argv[0]->type == ESEXP_RES_STRING) {
+		/* This is a list of new labels, we should used these in case of passing in old names.
+		   This all is required only because backward compatibility. */
+		const char *new_labels[] = { "$Labelimportant", "$Labelwork", "$Labelpersonal", "$Labeltodo", "$Labellater", NULL};
+		const char *label;
+		int i;
+
+		label = argv[0]->value.string;
+
+		for (i = 0; new_labels [i]; i++) {
+			if (label && strcmp (new_labels [i] + 6, label) == 0) {
+				label = new_labels [i];
+				break;
+			}
+		}
+
 		if (p->source && p->uid && camel_folder_has_summary_capability (p->source))
-			camel_folder_set_message_user_tag (p->source, p->uid, "label", argv[0]->value.string);
+			camel_folder_set_message_user_flag (p->source, p->uid, label, TRUE);
 		else
-			camel_message_info_set_user_tag(p->info, "label", argv[0]->value.string);
-		camel_filter_driver_log (driver, FILTER_LOG_ACTION, "Set label to %s", argv[0]->value.string);
+			camel_message_info_set_user_flag (p->info, label, TRUE);
+		camel_filter_driver_log (driver, FILTER_LOG_ACTION, "Set label to %s", label);
 	}
 
 	return NULL;
@@ -1468,7 +1483,7 @@ camel_filter_driver_filter_message (CamelFilterDriver *driver, CamelMimeMessage
 			goto error;
 		case CAMEL_SEARCH_MATCHED:
 			filtered = TRUE;
-			camel_filter_driver_log (driver, FILTER_LOG_START, node->name);
+			camel_filter_driver_log (driver, FILTER_LOG_START, "%s", node->name);
 
 			if (camel_debug(":filter"))
 				printf("filtering '%s' applying rule %s\n",
